/* ------------------------------------------------------------------------------------------------ */
/* FILE :		Demo_0_LPC1227STK_IAR.c																*/
/*																									*/
/* CONTENS :																						*/
/*				Demo for the Olimex LPC1277-STK evaluation board									*/
/*				Flashing info and control shown on LEDs and the serial port							*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/*																									*/
/* Copyright (c) 2012-2013, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.3 $																				*/
/*	$Date: 2013/06/14 18:11:53 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_DEMO != 0
	#error "Wrong OS_DEMO value" 
#endif

#include <stdio.h>
#include <stdarg.h>
#include <Abassi.h>

/* ------------------------------------------------ */

#include "lpc12xx_libcfg_default.h"
#include "lpc12xx_gpio.h"
#include "lpc12xx_iocon.h"
#include "lpc12xx_sysctrl.h"
#include "lpc12xx_uart.h"

#define BAUDRATE	  115200						/* Serial port baud rate						*/
#define SYS_FREQ	24000000						/* Processor clock frequency					*/

#define TEST_UART      LPC_UART0
#define TEST_UART_RXD  IOCON_UART_RXD0_LOC0
#define TEST_UART_TXD  IOCON_UART_TXD0_LOC0

IOCON_PIO_CFG_Type PIO_mode;

void UARTInit(void);

/* ------------------------------------------------ */

SEM_t *SemLed0;										/* Sempahore TaskLed0 blocks on					*/
SEM_t *SemLed1;										/* Sempahore TaskLed1 blocks on					*/
SEM_t *SemLed2;										/* Sempahore TaskLed2 blocks on					*/

TSK_t *TaskLed0;									/* Task toggling LED 0							*/
TSK_t *TaskLed1;									/* Task toggling LED 1							*/
TSK_t *TaskLed2;									/* Task toggling LED 2							*/

int    Debounce[3];									/* Switch debouncing counters					*/

static volatile int g_Rate;							/* Controils the LEDs toggling rate				*/

const char Running[] = "|/-\\";

char Flashing[] = "[ ]  [ ]  [ ]";

/* ------------------------------------------------ */

void FctLed0(void);									/* Function attached to TaskLed0				*/
void FctLed1(void);									/* Function attached to TaskLed1				*/
void FctLed2(void);									/* Function attached to TaskLed2				*/
int  GetKey(void);
void SwitchISR(void);								/* interrupt handler for the switches			*/

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int main(void)
{
int  ii;											/* General purpose								*/
int  jj;											/* General purpose								*/
int  Prio = 0;										/* Priority of the SysTick interrupt			*/
int *TimerBase;										/* SysTick register base address				*/


/* ------------------------------------------------ */
/* 	Processor clock set-up							*/

	SystemInit();

/* ------------------------------------------------ */
/* 	Start the RTOS									*/

	OSstart();

/* ------------------------------------------------ */
/* UART set-up										*/

	UARTInit();

/* ------------------------------------------------ */
/* SysTick Set-up									*/

	TimerBase = (int *)0xE000E000;
	((volatile int *)0xE000Ed20)[0] = (((volatile int *)0xE000Ed20)[0] & 0x1FFFFFFF)
	                                |   ((Prio & 7) << 29);
	OSisrInstall(-1, TIMtick);						/* Install the OS timer tick function			*/
	TimerBase[5] = (((SYS_FREQ/100000L) * OS_TIMER_US)/10)-1;
	TimerBase[4] = 0x7;								/* internal clock / int & enable				*/

/* ------------------------------------------------ */
/* GPIO (Switches) set-up							*/

    SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO2, ENABLE);
    NVIC_EnableIRQ(EINT2_IRQn);

    IOCON_StructInit(&PIO_mode);

	PIO_mode.type = IOCON_PIO_2_12;					/* USER1 switch in input						*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO2, 12, 0);
	GPIO_SetLowLevel(LPC_GPIO2, 12, 1);
	GPIO_IntSetType( LPC_GPIO2, 12, GPIO_INTERRUPT_FALLING);
	GPIO_IntSetMask( LPC_GPIO2, 12, 1 );

	PIO_mode.type = IOCON_PIO_2_11;					/* USER2 switch in input						*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO2, 11, 0);
	GPIO_SetLowLevel(LPC_GPIO2, 11, 1);
	GPIO_IntSetType( LPC_GPIO2, 11, GPIO_INTERRUPT_FALLING);
	GPIO_IntSetMask( LPC_GPIO2, 11, 1 );

	PIO_mode.type = IOCON_PIO_2_10;					/* USER3 switch in input						*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO2, 10, 0);
	GPIO_SetLowLevel(LPC_GPIO2, 10, 1);
	GPIO_IntSetType( LPC_GPIO2, 10, GPIO_INTERRUPT_FALLING);
	GPIO_IntSetMask( LPC_GPIO2, 10, 1 );

	OSisrInstall(27, SwitchISR);					/* The 3 switches are mapped on same interrupt	*/

/* ------------------------------------------------ */
/* LED set-up										*/

	PIO_mode.type = IOCON_PIO_1_5;					/* LED1 in output								*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO1, 5, 1);
	GPIO_SetHighLevel(LPC_GPIO1, 5, 1);

	PIO_mode.type = IOCON_PIO_1_4;					/* LED2 in output								*/
	IOCON_SetFunc(&PIO_mode);
	GPIO_SetDir(LPC_GPIO1, 4, 1);
	GPIO_SetHighLevel(LPC_GPIO1, 4, 1);

/* ------------------------------------------------ */
/* Application set-up								*/

	TaskLed0 = TSKcreate("LED 0", 1, TC_TSK_STACK, &FctLed0, 1);
	TaskLed1 = TSKcreate("LED 1", 1, TC_TSK_STACK, &FctLed1, 1);
	TaskLed2 = TSKcreate("LED 2", 1, TC_TSK_STACK, &FctLed2, 1);

	SemLed0  = SEMopen("LED 0");
	SemLed1  = SEMopen("LED 1");
	SemLed2  = SEMopen("LED 2");

	Debounce[0] = 0;
	Debounce[1] = 0;
	Debounce[2] = 0;

/* ------------------------------------------------ */

	puts("\n\n");									/* Welcome banner								*/
	puts("\n           Abassi Demo 0");
	puts("\n          Olimex LPC-1227\n");
	puts("\n         Use the 3 Buttons\n");
	puts("\n       or + - on serial port\n\n");

/* ------------------------------------------------ */
/* Interrupts ON, the application runs				*/

	g_Rate = 5;

	OSeint(1);

 	TSKsetPrio(TSKgetID(""), OS_PRIO_MIN);			/* No IdleTask created, Adam-Eve is Idle Task	*/

	jj = 0;											/* From here on, the code is the Idle Task		*/
   	for(;;) {										/* Check key to be pressed to change the rate	*/
		switch(GetKey()) {
		case '+':									/* Key '+' pressed, increase the toggling rate	*/
			ii = g_Rate - 1;						/* which means to decrease the variable g_Rate	*/
			if (ii <= 0) {							/* Floor g_Rate at 0							*/
				ii = 1;
			}
			g_Rate = ii;
			break;
		case '-':									/* Key '-' pressed, decrease the toggling rate	*/
			ii = g_Rate + 1;						/* which means to increase the variable g_Rate	*/
			if (ii > 10) {							/* Ceiling g_Rate at 10							*/
				ii = 10;
			}
			g_Rate = ii;;
			break;
		default:									/* Any other key is ignored						*/
			break;
		}

		ii = g_Rate;								/* Update the info on the serial port			*/
		printf("%c  Current rate %d%s  %s\r", Running[jj++], ii, (ii<10)?" ":"", Flashing);

		jj &= 3;
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Idle Task																						*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void IdleTask(void)
{
	TSKselfSusp();									/* A&E is set to Idle priority so we can		*/
													/* disappear									*/
	for(;;);
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* LED0 task																						*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctLed0(void)
{
	for ( ; ; ) {
		SEMwaitBin(SemLed0, -1);					/* Wait for ISR to post SemLed0					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[1] = '*';							/* Show on serial port LED 0 on					*/
		GPIO_SetLowLevel(LPC_GPIO1, 5, 1);

		SEMwaitBin(SemLed0, -1);					/* Wait for ISR to post SemLed0					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[1] = ' ';							/* Show on serial port LED 0 off				*/
		GPIO_SetHighLevel(LPC_GPIO1, 5, 1);
	}
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* LED1 task																						*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctLed1(void)
{
	for ( ; ; ) {
		SEMwaitBin(SemLed1, -1);					/* Each posting of SemLed1 toggle LED 1			*/
		Flashing[6] = '*';

		SEMwaitBin(SemLed1, -1);					/* Each posting of SemLed1 toggle LED 1			*/
		Flashing[6] = ' ';
	}
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* LED2 task																						*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctLed2(void)
{
	for ( ; ; ) {
		SEMwaitBin(SemLed2, -1);					/* Wait for ISR to post SemLed2					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[11] = '*';							/* Show on serial port LED 2 on					*/
		GPIO_SetLowLevel(LPC_GPIO1, 4, 1);

		SEMwaitBin(SemLed2, -1);					/* Wait for ISR to post SemLed2					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[11] = ' ';							/* Show on serial port LED 2 off				*/
		GPIO_SetHighLevel(LPC_GPIO1, 4, 1);
	}
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* UAER1 / UAR2 / UAR3  switch interrupt function													*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void SwitchISR(void)
{

	if ((GPIO_IntGetMaskStatus(LPC_GPIO2, 12))		/* USER 1 button								*/
	&&  (Debounce[0] == 0)) {						/* Speed up the flashing rate					*/
		if (--g_Rate <= 0) {
			g_Rate = 1;
		}
		Debounce[0] = OS_TICK_PER_SEC/50;
        GPIO_IntClear(LPC_GPIO2, 12, 1 );
	}

	if ((GPIO_IntGetMaskStatus(LPC_GPIO2, 11))		/* USER 2 button								*/
	&&  (Debounce[1] == 0)) {						/* Set flashing rate to middle					*/
		g_Rate = 5;
		Debounce[1] = OS_TICK_PER_SEC/50;
        GPIO_IntClear(LPC_GPIO2, 11, 1 );
	}

	if ((GPIO_IntGetMaskStatus(LPC_GPIO2, 10))		/* USER 3 button								*/
	&&  (Debounce[0] == 0)) {						/* Slow down flashing rate						*/
		if (++g_Rate > 10) {
			g_Rate = 10;
		}
		Debounce[0] = OS_TICK_PER_SEC/50;
        GPIO_IntClear(LPC_GPIO2, 10, 1 );
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Timer call back function																			*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void TIMcallBack(void)
{
static int Count0=0;
static int Count2=0;


	if (++Count0 >= g_Rate*(OS_TICK_PER_SEC/10)) {	/* Determine if it's time to toggle LED 0		*/
		SEMpost(SemLed0);							/* When it is, post the semaphore TaslLed0 is	*/
		Count0 = 0;									/* blocked on									*/
	}
	if (++Count2 >= g_Rate*((81*OS_TICK_PER_SEC)/1000)) {/* Determine if it's time to toggle LED 2	*/
		SEMpost(SemLed2);							/* When it is, post the semaphore TaslLed2 is	*/
		Count2 = 0;									/* blocked on									*/
	}

	if (GPIO_GetPinValue(LPC_GPIO2, 12)				/* Switch debouncing							*/
	&&  (Debounce[0] != 0)) {
		Debounce[0]--;
	}

	if (GPIO_GetPinValue(LPC_GPIO2, 11)				/* Switch debouncing							*/
	&&  (Debounce[1] != 0)) {
		Debounce[1]--;
	}
	if (GPIO_GetPinValue(LPC_GPIO2, 10)				/* Switch debouncing							*/
	&&  (Debounce[2] != 0)) {
		Debounce[2]--;
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void UARTInit (void)
{
UART_CFG_Type      UARTConfigStruct;


	IOCON_StructInit(&PIO_mode);
	PIO_mode.type = TEST_UART_RXD;
	IOCON_SetFunc(&PIO_mode);
	PIO_mode.type = TEST_UART_TXD;
	IOCON_SetFunc(&PIO_mode);
  
	SYS_ResetPeripheral(SYS_PRESETCTRL_UART0_RST,DISABLE);
	SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_UART0, ENABLE); 
	SYS_SetUART0ClockDiv(1);

	UART_Init(TEST_UART);

	UART_GetConfig(TEST_UART, &UARTConfigStruct);
	UARTConfigStruct.baudrate = BAUDRATE;
	UARTConfigStruct.databits = UART_CFG_DATABIT_8;

	UART_SetConfig(TEST_UART, &UARTConfigStruct);
	UART_ConfigTXD(TEST_UART, ENABLE);

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{
size_t nCh;

	nCh = 0;
	for ( ; Bufsize > 0; --Bufsize) {
		if (*Buf == '\n') {
			UART_Send(LPC_UART0, (unsigned char *)"\r", 1, -1);
		}
		UART_Send(LPC_UART0, (unsigned char *)Buf++, 1, -1);
		nCh++;
	}

	return(nCh);
}

/* ------------------------------------------------------------------------------------------------ */

size_t __read(int handle, unsigned char * buffer, size_t size)
{
int RetVal;

	RetVal = UART_Receive(LPC_UART0, (unsigned char *)buffer, 1, -1);

	return(RetVal);
}


/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
unsigned char ccc;
int           RetVal;


	RetVal = UART_Receive(LPC_UART0, &ccc, 1, 1);
	if (RetVal != 0) {
		RetVal = (int)ccc;
	}
	return(RetVal);
}

/* EOF */
